package com.sd.business.service.in.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.in.dto.InOutStorageBomDto;
import com.sd.business.entity.in.dto.InOutStorageBomSelectDto;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.po.InOutStorage;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.in.vo.InOutStorageBomErpExportVo;
import com.sd.business.entity.in.vo.InOutStorageBomVo;
import com.sd.business.entity.in.vo.InOutStorageErpExportVo;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.warehouse.po.Warehouse;
import com.sd.business.mapper.in.InOutStorageBomMapper;
import com.sd.business.service.in.InOutStorageBomService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.framework.util.TemplateExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static java.util.Comparator.comparing;


/**
 * <p>
 * 出入库bom 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Service
public class InOutStorageBomServiceImpl extends ServiceImpl<InOutStorageBomMapper, InOutStorageBom> implements InOutStorageBomService {

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private HttpServletResponse response;

    @Override
    public Page<InOutStorageBomVo> getPage(InOutStorageBomSelectDto dto) {
        IWrapper<InOutStorageBom> wrapper = getWrapper();
        wrapper.orderByDesc("iosb", InOutStorageBom::getId);
        wrapper.like("ios", InOutStorage::getCode, dto.getCode());
        wrapper.like("p", Purchase::getCode, dto.getPurchaseCode());
        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.eq("ios", InOutStorage::getDepartmentId, dto.getDepartmentId());
        wrapper.eq("w", Warehouse::getId, dto.getWarehouseId());
        wrapper.eq("ios", InOutStorage::getType, dto.getType());
        wrapper.eq("ios", InOutStorage::getDetailType, dto.getDetailType());
        wrapper.ge("ios", InOutStorage::getCreateTime, dto.getBeginTime());
        wrapper.le("ios", InOutStorage::getCreateTime, dto.getEndTime());

        Page<InOutStorageBomVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public InOutStorageBomVo detail(Long id) {
        InOutStorageBom InOutStorageBom = this.getById(id);
        InOutStorageBomVo result = BeanUtil.toBean(InOutStorageBom, InOutStorageBomVo.class);
        return result;
    }

    @Override
    public void add(InOutStorageBomDto inOutStorageBomDto) {
        this.save(inOutStorageBomDto);
    }

    @Override
    public void edit(InOutStorageBomDto inOutStorageBomDto) {
        this.updateById(inOutStorageBomDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public void exportExcel(InOutStorageBomSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<InOutStorageBomVo> page = this.getPage(dto);
        List<InOutStorageBomVo> list = page.getRecords();
        // 查询详情信息
        Set<Long> inOutStorageIds = list.stream().map(InOutStorageBom::getInOutStorageId).collect(Collectors.toSet());
        if (ObjectUtil.isEmpty(inOutStorageIds)) {
            inOutStorageIds.add(-1L);
        }
        List<InOutStorage> inOutStorageList = inOutStorageService.list(q -> q
                .in(BaseIdPo::getId, inOutStorageIds)
                .orderByAsc(InOutStorage::getCreateTime));

        List<InOutStorageErpExportVo> inOutStorageErpExportVoList = new ArrayList<>();
        int codeNum = 1;
        String dateFormat = "";
        for (InOutStorage inOutStorage : inOutStorageList) {
            InOutStorageErpExportVo inOutStorageErpExportVo = new InOutStorageErpExportVo();
            inOutStorageErpExportVo.setId(inOutStorage.getId());
            inOutStorageErpExportVo.setPlantCode("SDTY");
            if (Objects.equals(inOutStorage.getType(), InOutTypeEnum.OUT.getKey())) {
                inOutStorageErpExportVo.setDocCode("T118");
            } else {
                inOutStorageErpExportVo.setDocCode("T119");
            }
            // 获取单号
            String format = DateUtil.format(inOutStorage.getCreateTime(), "-yyyyMM");
            if (!Objects.equals(dateFormat, format)) {
                dateFormat = format;
                codeNum = 1;
            }
            String code = inOutStorageErpExportVo.getDocCode() + format + String.format("%04d", codeNum);
            inOutStorageErpExportVo.setCode(code);
            inOutStorageErpExportVo.setCreateTime(DateUtil.format(inOutStorage.getCreateTime(), "yyyy/MM/dd HH:mm:ss"));
            inOutStorageErpExportVo.setDeptCode("GPIT");
            inOutStorageErpExportVo.setEmployeeCode("GPIT02");

            inOutStorageErpExportVoList.add(inOutStorageErpExportVo);
            codeNum++;
        }

        Map<Long, String> map = inOutStorageErpExportVoList.stream()
                .collect(Collectors.toMap(InOutStorageErpExportVo::getId, InOutStorageErpExportVo::getCode));
        Map<String, Integer> serialNumberMap = new HashMap<>();
        List<InOutStorageBomErpExportVo> inOutStorageBomErpExportVoList = list.stream().map(item -> {
            InOutStorageBomErpExportVo inOutStorageBomErpExportVo = new InOutStorageBomErpExportVo();
            inOutStorageBomErpExportVo.setCode(map.get(item.getInOutStorageId()));
            inOutStorageBomErpExportVo.setBomSpecCode(item.getBomSpecCode());
            inOutStorageBomErpExportVo.setQuantity(item.getQuantity());
            inOutStorageBomErpExportVo.setWarehouseCode(item.getWarehouseCode());
            inOutStorageBomErpExportVo.setUnit("PCS");
            inOutStorageBomErpExportVo.setPieces("0");
            inOutStorageBomErpExportVo.setSourceRtk("OTHER");
            inOutStorageBomErpExportVo.setUnitCost(BigDecimal.ZERO);
            inOutStorageBomErpExportVo.setCostAmt(BigDecimal.ZERO);
            return inOutStorageBomErpExportVo;
        })
                .sorted(comparing(InOutStorageBomErpExportVo::getCode))
                .peek(item -> {
                    Integer serialNumber = serialNumberMap.get(item.getCode());
                    if (serialNumber == null) {
                        serialNumber = 1;
                        serialNumberMap.put(item.getCode(), serialNumber);
                    } else {
                        serialNumberMap.put(item.getCode(), serialNumber+=1);
                    }
                    item.setSerialNumber(serialNumber);
                }).collect(Collectors.toList());
        String fileName;
        if (Objects.equals(dto.getType(), 1)) {
            fileName = "入库单";
        } else {
            fileName = "出库单";
        }
        TemplateExcelUtil.writeBrowserMultiSheet("inOutStorageDetails.xlsx", 2, fileName, response, inOutStorageErpExportVoList, inOutStorageBomErpExportVoList);
    }

}
