package com.sd.business.service.in.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.excel.enums.ExcelTypeEnum;
import com.sd.business.entity.in.dto.PurchaseWarehousingDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.po.InOutStorage;
import com.sd.business.entity.in.vo.PurchaseWarehousingVo;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.supplier.po.Supplier;
import com.sd.business.mapper.in.PurchaseWarehousingServiceMapper;
import com.sd.business.service.excel.ExcelGenerateLogService;
import com.sd.business.service.in.PurchaseWarehousingService;
import com.sd.business.strategy.impl.DefaultExportStrategy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Service
public class PurchaseWarehousingServiceImpl implements PurchaseWarehousingService {

    @Resource
    private PurchaseWarehousingServiceMapper purchaseWarehousingServiceMapper;

    @Autowired
    private ExcelGenerateLogService excelGenerateLogService;

    @Override
    public Page<PurchaseWarehousingVo> getPage(PurchaseWarehousingDto dto) {
        return purchaseWarehousingServiceMapper.getPage(dto.getPage(), getWrapper(dto));
    }

    @Override
    public void exportExcel(PurchaseWarehousingDto dto) {
        DefaultExportStrategy<PurchaseWarehousingVo> defaultExportStrategy = new DefaultExportStrategy<>(
                () -> getList(dto), PurchaseWarehousingVo.class, "采购入库");

        excelGenerateLogService.generateExcel(ExcelTypeEnum.PURCHASE_WAREHOUSING, defaultExportStrategy,
                DateUtil.formatDate(new Date()) + " 采购入库数据");
    }

    private List<PurchaseWarehousingVo> getList(PurchaseWarehousingDto dto) {
        List<PurchaseWarehousingVo> list = purchaseWarehousingServiceMapper.getList(getWrapper(dto));
        for (int i = 0; i < list.size(); i++) {
            PurchaseWarehousingVo purchaseWarehousingVo = list.get(i);
            purchaseWarehousingVo.setSerialNumber(i + 1);
        }

        return list;
    }

    private IWrapper<Object> getWrapper(PurchaseWarehousingDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        wrapper.eq("ios", InOutStorage::getType, InOutTypeEnum.IN.getKey());
        wrapper.eq("ios", InOutStorage::getDetailType, InDetailTypeEnum.PURCHASE.getKey());
        wrapper.like("p", Purchase::getCode, dto.getPurchaseCode());
        wrapper.like("s", Supplier::getName, dto.getSupplierName());
        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.ge("ios", InOutStorage::getCreateTime, dto.getBeginTime());
        wrapper.le("ios", InOutStorage::getCreateTime, dto.getEndTime());
        return wrapper;
    }

}
