package com.sd.business.service.order;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.core.service.BaseService;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderSelectDto;
import com.sd.business.entity.order.dto.SkuSpecPriceDto;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.vo.CompareVo;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.SkuSpecPriceVo;
import com.sd.business.entity.statement.vo.StatementOrderClassifyTotalCountVo;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 订单 服务类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
public interface OrderService extends BaseService<OrderInfo> {

    /**
     * 订单分页
     */
    Page<OrderInfoVo> getPage(OrderSelectDto dto);

    /**
     * 订单明细
     */
    OrderInfoVo detail(Long id);

    /**
     * 订单新增
     */
    void add(OrderInfoDto orderDto);

    /**
     * 订单编辑
     */
    void edit(OrderInfoDto orderDto);

    /**
     * 订单确认
     */
    void confirmation(OrderInfoDto orderDto);

    /**
     * 订单删除
     */
    void delete(Long id, boolean saveLog);

    /**
     * 订单删除
     */
    void delete(Long id);

    /**
     * 通过sku规格id获取产品价格
     */
    SkuSpecPriceVo getSkuSpecPrice(SkuSpecPriceDto dto);

    /**
     * 获取异常类型
     */
    String addExceptionType(String exceptionTypeStr, Integer addExceptionType);

    /**
     * 获取已删除订单
     */
    Page<OrderInfoVo> deletedOrderPage(OrderSelectDto dto);

    /**
     * 已删除订单明细
     */
    OrderInfoVo deletedOrderDetail(BaseSelectDto dto);

    /**
     * 转为正常订单
     */
    void turnToRegularOrder(Long id);

    /**
     * 订单删除并回滚库存
     */
    void deleteAndStore(Long id);

    /**
     * 订单对比
     */
    CompareVo compare(OrderSelectDto dto);

    /**
     * 获取订单bom的出库数量
     *
     * @param bomSpecId bom规格id
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    BigDecimal getOrderBomSpecOutStorageQuantity(Long bomSpecId, Date startDate, Date endDate);

    /**
     * 获取万里牛单号
     */
    String getOrderWlnCodeStr(OrderSelectDto dto);

    /**
     * 订单是否存在包材
     *
     * @param orderId
     * @return
     */
    Boolean isExistOrderSkuBom(Long orderId);

    /**
     * 获取订单送货单号
     * @return
     */
    String getOrderDeliveryCode();

    /**
     * 保存订单送货单号
     * @param dto
     */
    void saveOrderDeliveryCode(OrderInfoDto dto);

    /**
     * 挂起订单
     * @param id
     */
    void suspendOrder(Long id);

    /**
     * 取消挂起订单
     * @param id
     */
    void cancelSuspendOrder(Long id);

    /**
     * 获取订单分类数量总计
     * @return
     */
    StatementOrderClassifyTotalCountVo getOrderClassifyTotalCount(List<Long> statementIds);
}
