package com.sd.business.service.production.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.order.dto.OrderSkuDto;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderPackageBom;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuProductionCost;
import com.sd.business.entity.production.dto.ProductionExceedReceiveDto;
import com.sd.business.entity.production.dto.ProductionOrderDto;
import com.sd.business.entity.production.dto.RapidPackagingDto;
import com.sd.business.entity.production.dto.SetTagDto;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.production.vo.ProductionOrderDetailVo;
import com.sd.business.entity.production.vo.ProductionOrderScheduleVo;
import com.sd.business.entity.production.vo.ProductionOrderVo;
import com.sd.business.entity.statement.dto.StatementOfAccountDto;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.production.ProductionOrderMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.order.OrderPackageBomService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuProductionCostService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionOrderService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.framework.util.Assert;
import com.sd.mq.enums.WorkOrderStatusEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class ProductionOrderServiceImpl implements ProductionOrderService {

    @Autowired
    private ProductionOrderMapper productionOrderMapper;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private OrderSkuProductionCostService orderSkuProductionCostService;

    @Autowired
    private InventoryFinishedService inventoryFinishedService;

    @Autowired
    private ProductionWorkOrderService productionWorkOrderService;

    @Override
    public Page<ProductionOrderVo> getPage(ProductionOrderDto dto) {

        IWrapper<ProductionOrderVo> wrapper = IWrapper.getWrapper();
        wrapper.ge("oi", OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey());
        wrapper.like("oi", OrderInfo::getCode, dto.getCode());
        wrapper.like("oi", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("oi", OrderInfo::getDepartmentId, dto.getDepartmentId());
        wrapper.in("oi", OrderInfo::getTag, dto.getTagList());
        wrapper.ge("oi", OrderInfo::getDeliveryTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getDeliveryTime, dto.getEndTime());
        wrapper.eq("oi", OrderInfo::getStatus, dto.getStatus());

        Page<ProductionOrderVo> page = productionOrderMapper.getPage(dto.getPage(), wrapper);
        List<ProductionOrderVo> records = page.getRecords();
        if (records.isEmpty()) {
            return page;
        }

        // 赋值进度
        List<Long> orderIdList = records.stream().map(ProductionOrderVo::getOrderId).collect(Collectors.toList());
        Map<Long, List<ProductionOrderScheduleVo>> map = productionWorkOrderService.getProductionOrderScheduleMap(orderIdList);
        for (ProductionOrderVo record : records) {
            record.setProductionOrderScheduleVoList(map.getOrDefault(record.getOrderId(), Collections.emptyList()));
        }

        return page;
    }

    @Override
    public void setTag(SetTagDto dto) {
        orderService.update(q -> q.in(BaseIdPo::getId, dto.getOrderIdList()).set(OrderInfo::getTag, dto.getTag()));
    }

    @Override
    public ProductionOrderDetailVo detail(BaseSelectDto dto) {
        Long id = dto.getId();
        Assert.notNull(id, "订单id不能为空");

        List<ProductionOrderDetailVo.SkuSpec> skuSpecList = getSkuSpecList(id);
        List<ProductionOrderDetailVo.OrderPackage> packageList = getPackageList(id);

        ProductionOrderDetailVo vo = new ProductionOrderDetailVo();
        vo.setSkuSpecList(skuSpecList);
        vo.setPackageList(packageList);
        return vo;
    }

    @Override
    public synchronized void rapidPackaging(RapidPackagingDto dto) {
        Long orderId = dto.getOrderId();
        OrderInfo orderInfo = orderService.getById(orderId);
        if (!orderInfo.getStatus().equals(OrderStatusEnum.IN_PRODUCTION.getKey())) {
            throw new ServiceException("订单已被处理");
        }

        List<OrderPackageBom> orderPackageBomList = dto.getPackageList();
        orderPackageBomList.forEach(item -> item.setOrderId(orderId));
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, orderId);

        Date date = new Date();
        orderService.update(q -> q
                .eq(BaseIdPo::getId, orderId)
                .set(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey())
                .set(OrderInfo::getShippingTime, date)
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, date));

        // 订单生产完成时，一键完成生产任务
        productionWorkOrderService.update(q -> q
                .eq(ProductionWorkOrder::getOrderId, orderId)
                .notIn(ProductionWorkOrder::getStatus, WorkOrderStatusEnum.PRODUCTION_ANOMALY.getKey(), WorkOrderStatusEnum.RESCHEDULE.getKey())
                .set(ProductionWorkOrder::getStatus, WorkOrderStatusEnum.PRODUCTION_COMPLETION.getKey())
                .set(ProductionWorkOrder::getCompleteTime, date)
                .set(BasePo::getUpdateTime, date)
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId()));

        // 生成对账单
        StatementOfAccountDto statement = new StatementOfAccountDto();
        statement.setDepartmentId(orderInfo.getDepartmentId());
        statement.setOrderIdList(Collections.singletonList(orderInfo.getId()));
        statementOfAccountService.add(statement);

        // 生产入库
        inventoryFinishedService.productionWarehousing(Collections.singletonList(orderId));

        // 库存销售出库
        inventoryFinishedService.saleOutOfWarehouse(Collections.singletonList(orderId));

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void exceedReceive(ProductionExceedReceiveDto dto) {
        Long orderId = dto.getOrderId();
        OrderInfo orderInfo = orderService.getById(orderId);
        if (orderInfo == null) {
            throw new ServiceException("未知订单");
        }
        if (!Objects.equals(orderInfo.getStatus(), OrderStatusEnum.IN_PRODUCTION.getKey())) {
            throw new ServiceException("操作失败，请选择生产中的订单");
        }
        List<OrderSkuDto> productionCostList = dto.getProductionCostList();
        List<OrderSkuDto> exceedReceiveSkuList = productionCostList.stream()
                .filter(item -> item.getQuantity() != null
                        && item.getQuantity().compareTo(BigDecimal.ZERO) > 0)
                .collect(Collectors.toList());
        if (exceedReceiveSkuList.isEmpty()) {
            throw new ServiceException("操作失败，订单sku超领明细列表不能为空");
        }
        Map<Long, BigDecimal> map = exceedReceiveSkuList.stream().collect(Collectors.toMap(
                OrderSkuDto::getBomSpecId, OrderSkuDto::getQuantity, BigDecimal::add));
        boolean result = inventoryService.lockStorage(map, WarehouseConstant.SEMI_FINISHED_PRODUCT);
        if (!result) {
            throw new ServiceException("库存不足，超领失败！");
        }
        // 出入库信息
        List<InOutStorageBom> inOutStorageBomList = map.keySet().stream().map(item -> {
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(item);
            inOutStorageBom.setQuantity(map.get(item));
            return inOutStorageBom;
        }).collect(Collectors.toList());
        // 出库
        InOutStorageDto lockPurchaseDefective = new InOutStorageDto();
        lockPurchaseDefective.setType(InOutTypeEnum.OUT.getKey());
        lockPurchaseDefective.setDetailType(OutDetailTypeEnum.PRODUCTION.getKey());
        lockPurchaseDefective.setWarehouseId(WarehouseConstant.SEMI_FINISHED_PRODUCT);
        lockPurchaseDefective.setDepartmentId(DepartmentConstant.SD_SPORTS);
        lockPurchaseDefective.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        lockPurchaseDefective.setRemark("订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + "超领出库");
        lockPurchaseDefective.setLockStorage(StatusConstant.YES);
        lockPurchaseDefective.setInOutStorageBomList(JSON.parseArray(JSON.toJSONString(inOutStorageBomList), InOutStorageBom.class));
        inOutStorageService.add(lockPurchaseDefective);
        // 入库
        InOutStorageDto inStorageDto = new InOutStorageDto();
        inStorageDto.setType(InOutTypeEnum.IN.getKey());
        inStorageDto.setDetailType(InDetailTypeEnum.ABANDON.getKey());
        // 制作损坏入库到生产次品仓，裸垫质量不良入库到采购次品仓
        if (Objects.equals(dto.getExceedReceiveReason(), 1)) {
            inStorageDto.setWarehouseId(WarehouseConstant.PRODUCTION_DEFECTIVE);
        } else {
            inStorageDto.setWarehouseId(WarehouseConstant.PURCHASE_DEFECTIVE);
        }
        inStorageDto.setDepartmentId(DepartmentConstant.SD_SPORTS);
        inStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        inStorageDto.setRemark("订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + "超领报废入库");
        inStorageDto.setInOutStorageBomList(inOutStorageBomList);
        inOutStorageService.add(inStorageDto);
        // 人为损坏, 新增订单生产成本的材料成本
        if (Objects.equals(dto.getExceedReceiveReason(), 1)) {
            List<Long> bomSpecIds = exceedReceiveSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
            List<Long> orderSkuIds = exceedReceiveSkuList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
            Map<Long, BomSpec> bomSpecMap = bomSpecService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, bomSpecIds));
            Map<Long, OrderSkuProductionCost> orderSkuProductionCostMap = orderSkuProductionCostService.mapKEntity(
                    OrderSkuProductionCost::getOrderSkuId,
                    q -> q.eq(OrderSkuProductionCost::getOrderId, orderId)
                            .in(OrderSkuProductionCost::getOrderSkuId, orderSkuIds)
            );
            for (OrderSkuDto orderSkuDto : exceedReceiveSkuList) {
                BomSpec bomSpec = bomSpecMap.get(orderSkuDto.getBomSpecId());
                OrderSkuProductionCost orderSkuProductionCost = orderSkuProductionCostMap.get(orderSkuDto.getId());
                // 计算当前sku超领的材料成本
                BigDecimal materialCost = orderSkuDto.getQuantity().multiply(bomSpec.getInternalSellingPrice());

                orderSkuProductionCost.setMaterialCost(orderSkuProductionCost.getMaterialCost().add(materialCost));
            }
            // 更新订单生产成本的材料成本
            orderSkuProductionCostService.updateBatchById(new ArrayList<>(orderSkuProductionCostMap.values()));
        }
    }

    private List<ProductionOrderDetailVo.SkuSpec> getSkuSpecList(Long orderId) {

        // 获取订单sku规格
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderId));

        // 相同订单sku规格合并，数量相加
        List<ProductionOrderDetailVo.SkuSpec> skuSpecList = new ArrayList<>(
                orderSkuList.stream()
                        .map(item -> {
                            ProductionOrderDetailVo.SkuSpec skuSpec = new ProductionOrderDetailVo.SkuSpec();
                            skuSpec.setSkuSpecId(item.getSkuSpecId());
                            skuSpec.setQuantity(item.getQuantity());
                            return skuSpec;
                        })
                        .collect(Collectors.toMap(
                                ProductionOrderDetailVo.SkuSpec::getSkuSpecId,
                                Function.identity(),
                                (v1, v2) -> {
                                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                    return v1;
                                })
                        ).values());

        // 赋值订单sku规格品号以及品名
        skuSpecService.attributeAssign(skuSpecList, ProductionOrderDetailVo.SkuSpec::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecName(skuSpec.getName());
            item.setSkuSpecCode(skuSpec.getCode());
        });

        return skuSpecList;
    }

    private List<ProductionOrderDetailVo.OrderPackage> getPackageList(Long orderId) {

        List<OrderPackageBom> list = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, orderId));

        List<ProductionOrderDetailVo.OrderPackage> orderPackageList = list.stream().map(item -> {
            ProductionOrderDetailVo.OrderPackage orderPackage = new ProductionOrderDetailVo.OrderPackage();
            orderPackage.setBomSpecId(item.getBomSpecId());
            orderPackage.setQuantity(item.getQuantity());
            orderPackage.setCostPrice(item.getCostPrice());
            orderPackage.setInternalSellingPrice(item.getInternalSellingPrice());
            return orderPackage;
        }).collect(Collectors.toList());

        // 赋值品名 品号 销售单价 成本价
        bomSpecService.attributeAssign(orderPackageList, ProductionOrderDetailVo.OrderPackage::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        return orderPackageList;
    }

}
