package com.sd.business.service.purchase.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.dto.BomSpecSelectDto;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.bom.vo.BomSpecVo;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.purchase.dto.PurchaseBoardSelectDto;
import com.sd.business.entity.purchase.dto.PurchaseBomDetailsSelectDto;
import com.sd.business.entity.purchase.dto.PurchaseBomSelectDto;
import com.sd.business.entity.purchase.dto.PurchaseInTransitBomSelectDto;
import com.sd.business.entity.purchase.enums.PurchaseStatusEnum;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.purchase.po.PurchaseBom;
import com.sd.business.entity.purchase.vo.*;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.purchase.PurchaseBomMapper;
import com.sd.business.service.bom.BomClassifyService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.order.OrderSalesShipmentStatisticsService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.StreamUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 采购合同bom 服务实现类
 * </p>
 *
 * @author
 * @since 2023-08-07
 */
@Service
public class PurchaseBomServiceImpl extends ServiceImpl<PurchaseBomMapper, PurchaseBom> implements PurchaseBomService {

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private BomClassifyService bomClassifyService;

    @Autowired
    private InOutStorageService inOutStorageService;
    
    @Autowired
    private OrderService orderService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private OrderSalesShipmentStatisticsService orderSalesShipmentStatisticsService;

    @Override
    public Page<PurchaseBomSelectVo> getPage(PurchaseBomSelectDto dto) {
        Assert.notEmpty(dto.getPurchaseId(), "采购合同id不能为空");
        IWrapper<PurchaseBom> wrapper = getWrapper();
        wrapper.eq("pb", PurchaseBom::getPurchaseId, dto.getPurchaseId());
        wrapper.like("b", Bom::getName, dto.getBomName());
        wrapper.like("b", Bom::getCode, dto.getBomCode());
        wrapper.eq("b", Bom::getSpecies, dto.getSpecies());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("b", Bom::getEmbossingProcess, dto.getEmbossingProcess());
        wrapper.eq("b", Bom::getFrontGrain, dto.getFrontGrain());
        wrapper.eq("b", Bom::getReverseGrain, dto.getReverseGrain());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.eq("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.eq("bs", BomSpec::getName, dto.getBomSpecName());
        if (ObjectUtil.isNotNull(dto.getBomClassifyId())) {
            List<Long> childrenIdList = bomClassifyService.getChildrenIdList(dto.getBomClassifyId());
            wrapper.in("b", Bom::getBomClassifyId, childrenIdList);
        }
        if (ObjectUtil.isNotNull(dto.getBomClassifyIdList())) {
            List<Long> childrenIdList = bomClassifyService.getChildrenIdList(dto.getBomClassifyIdList());
            wrapper.in("b", Bom::getBomClassifyId, childrenIdList);
        }
        wrapper.orderByDesc("b", Bom::getId);
        wrapper.groupBy("b.id");
        Page<PurchaseBomSelectVo> page = this.baseMapper.getPurchaseBomPage(dto.getPage(), wrapper);
        List<PurchaseBomSelectVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        List<Long> bomIdList = records.stream().map(PurchaseBomSelectVo::getBomId).collect(Collectors.toList());
        wrapper = IWrapper.getWrapper();
        wrapper.eq("pb", PurchaseBom::getPurchaseId, dto.getPurchaseId());
        wrapper.in("bs", BomSpec::getBomId, bomIdList);
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.eq("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.eq("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.orderByDesc("pb", PurchaseBom::getId);
        List<PurchaseBomInfoVo> list = this.baseMapper.getPurchaseBomInfoList(wrapper);
        Map<Long, List<PurchaseBomInfoVo>> bomSpecIdMap = list.stream().collect(Collectors.groupingBy(PurchaseBomInfoVo::getBomId));

        for (PurchaseBomSelectVo record : records) {
            record.setBomSpecList(bomSpecIdMap.get(record.getBomId()));
        }
        return page;
    }

    @Override
    public PurchaseBomVo detail(Long id) {
        PurchaseBom PurchaseBom = this.getById(id);
        PurchaseBomVo result = BeanUtil.toBean(PurchaseBom, PurchaseBomVo.class);
        return result;
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public Page<PurchaseInTransitBomVo> getPurchaseInTransitBomPage(PurchaseInTransitBomSelectDto dto) {
        IWrapper<PurchaseInTransitBomVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("p", Purchase::getStatus, PurchaseStatusEnum.UNDER_PURCHASE.getKey());
        wrapper.like("p", Purchase::getCode, dto.getPurchaseCode());
        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.apply("pb.purchase_quantity > pb.arrival_quantity");
        wrapper.orderByAsc("bs", BomSpec::getCode)
                .orderByDesc("bs", BomSpec::getLength)
                .orderByAsc("bs", BomSpec::getWidth)
                .orderByDesc("bs", BomSpec::getHeight);
        return this.baseMapper.getPurchaseInTransitBomPage(dto.getPage(), wrapper);
    }

    @Override
    public Page<PurchaseBomBoardVo> getPurchaseBoardPage(PurchaseBoardSelectDto dto) {
        BomSpecSelectDto bomSpecSelectDto = new BomSpecSelectDto();
        bomSpecSelectDto.setName(dto.getBomSpecName());
        bomSpecSelectDto.setCode(dto.getBomSpecCode());
        bomSpecSelectDto.setPageNum(dto.getPageNum());
        bomSpecSelectDto.setPageSize(dto.getPageSize());
        Page<BomSpecVo> bomSpecVoPage = bomSpecService.getPage(bomSpecSelectDto);
        List<BomSpecVo> records = bomSpecVoPage.getRecords();
        Page<PurchaseBomBoardVo> page = new Page<>();
        page.setSize(bomSpecVoPage.getSize());
        page.setTotal(bomSpecVoPage.getTotal());
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        Date date = new Date();
        List<PurchaseBomBoardVo> list = new ArrayList<>();

        List<Long> bomSpecIds = records.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        // 查询库存数据
        Map<Long, List<Inventory>> inventorMap = inventoryService.mapKGroup(
                Inventory::getBomSpecId,
                q -> q.in(Inventory::getBomSpecId, bomSpecIds)
                        .eq(Inventory::getDepartmentId, DepartmentConstant.SD_SPORTS)
                        .in(Inventory::getWarehouseId,
                                Arrays.asList(WarehouseConstant.SEMI_FINISHED_PRODUCT, WarehouseConstant.PACKAGING_MATERIAL)));

        // 查询周转率数据
        Map<Long, BigDecimal> turnoverRateMap = orderSalesShipmentStatisticsService.getTurnoverRateMapByBomSpecIds(bomSpecIds);

        // 获取bom的下一批到货天数
        IWrapper<PurchaseBom> wrapper = IWrapper.getWrapper();
        wrapper.in("pb", PurchaseBom::getBomSpecId, bomSpecIds);
        wrapper.apply("pb.purchase_quantity > pb.arrival_quantity");
        wrapper.eq("p", Purchase::getStorageStatus, StatusConstant.NO);
        wrapper.gt("p", Purchase::getDeliveryDate, date);
        wrapper.groupBy("pb.bom_spec_id");
        List<PurchaseInTransitBomVo> deliveryDateList = this.baseMapper.getPurchaseBomDeliveryDate(wrapper);
        Map<Long, Date> deliveryDateMap = deliveryDateList.stream().collect(Collectors.toMap(PurchaseInTransitBomVo::getBomSpecId, PurchaseInTransitBomVo::getDeliveryDate));

        for (BomSpecVo record : records) {
            PurchaseBomBoardVo purchaseBomBoard = new PurchaseBomBoardVo();
            Long bomSpecId = record.getId();
            List<Inventory> inventoryList = inventorMap.get(bomSpecId);

            // 当前库存数
            BigDecimal inventoryQuantity = ObjectUtil.isEmpty(inventoryList) ? BigDecimal.ZERO : StreamUtil.bigDecimalAdd(inventoryList, Inventory::getQuantity);
            // 近7天消耗量
            BigDecimal quantitySevenDays = orderService.getOrderBomSpecOutStorageQuantity(record.getId(), DateUtil.offsetDay(date, -7), date);
            // 近30天消耗量
            BigDecimal quantityThirtyDays = orderService.getOrderBomSpecOutStorageQuantity(record.getId(), DateUtil.offsetDay(date, -30), date);

            // 40天安全库存
            BigDecimal safetyInventoryQuantity = quantityThirtyDays.divide(new BigDecimal("30"), 2, RoundingMode.HALF_UP)
                    .multiply(new BigDecimal("40")).setScale(0, RoundingMode.HALF_UP);

            // 近14天消耗量
            BigDecimal quantityTwoWeek = orderService.getOrderBomSpecOutStorageQuantity(record.getId(), DateUtil.offsetDay(date, -14), date);
            // 近两周减去近一周的消耗量为上周数量
            BigDecimal quantityLastWeek = quantityTwoWeek.subtract(quantitySevenDays);
            // 周环比率
            BigDecimal weekOnWeekRatio;
            if (quantityLastWeek.compareTo(BigDecimal.ZERO) == 0) {
                weekOnWeekRatio = quantitySevenDays;
            } else {
                weekOnWeekRatio = quantitySevenDays.subtract(quantityLastWeek).divide(quantityLastWeek, 2, RoundingMode.HALF_UP);
            }

            // 预计消耗天数
            int predictOutStorageDays = 0;
            if (quantitySevenDays.compareTo(BigDecimal.ZERO) != 0) {
                predictOutStorageDays = inventoryQuantity.divide(quantitySevenDays
                        .divide(new BigDecimal("7"), 2, RoundingMode.HALF_UP), 2, RoundingMode.HALF_UP).intValue();
            }

            // 获取在途总数
            purchaseBomBoard.setInTransitSum(this.getPurchaseBomInTransitSum(bomSpecId));

            // 获取下一批到货天数
            Date deliveryDate = deliveryDateMap.get(bomSpecId);
            long nextDeliveryDays = 0L;
            if (ObjectUtil.isNotEmpty(deliveryDate)) {
                nextDeliveryDays = DateUtil.between(date, deliveryDate, DateUnit.DAY);
            }

            // 赋值
            purchaseBomBoard.setBomSpecName(record.getName());
            purchaseBomBoard.setBomSpecCode(record.getCode());
            purchaseBomBoard.setColour(record.getColour());
            purchaseBomBoard.setLength(record.getLength());
            purchaseBomBoard.setHeight(record.getHeight());
            purchaseBomBoard.setWidth(record.getWidth());
            purchaseBomBoard.setBomSpecId(bomSpecId);
            purchaseBomBoard.setInventoryQuantity(inventoryQuantity);
            purchaseBomBoard.setPredictOutStorageDays(predictOutStorageDays);
            purchaseBomBoard.setSafetyInventoryQuantity(safetyInventoryQuantity);
            purchaseBomBoard.setOutStorageQuantitySevenDays(quantitySevenDays);
            purchaseBomBoard.setOutStorageQuantityThirtyDays(quantityThirtyDays);
            purchaseBomBoard.setOutStorageQuantityFifteenDays(orderService.getOrderBomSpecOutStorageQuantity(record.getId(),DateUtil.offsetDay(date, -15), date));
            purchaseBomBoard.setOutStorageQuantitySixtyDays(orderService.getOrderBomSpecOutStorageQuantity(record.getId(), DateUtil.offsetDay(date, -60), date));
            purchaseBomBoard.setOutStorageWeekOnWeekRatio(weekOnWeekRatio);
            purchaseBomBoard.setNextDeliveryDays((int) nextDeliveryDays);
            purchaseBomBoard.setTurnoverRate(turnoverRateMap.get(bomSpecId));
            list.add(purchaseBomBoard);
        }
        page.setRecords(list);
        return page;
    }

    @Override
    public Page<PurchaseBomDetailsVo> getDetailsPage(PurchaseBomDetailsSelectDto dto) {
        IWrapper<PurchaseBomDetailsVo> wrapper = IWrapper.getWrapper();;
        wrapper.like("p", Purchase::getCode, dto.getPurchaseCode());
        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.orderByDesc("p", Purchase::getId);
        return this.baseMapper.getDetailsPage(dto.getPage(), wrapper);
    }

    @Override
    public BigDecimal getPurchaseBomInTransitSum(Long bomSpecId) {
        IWrapper<PurchaseBom> wrapper = getWrapper();
        wrapper.eq("pb", PurchaseBom::getBomSpecId, bomSpecId);
        wrapper.eq("p", Purchase::getStatus, PurchaseStatusEnum.UNDER_PURCHASE.getKey());
        wrapper.apply("pb.purchase_quantity > pb.arrival_quantity");
        return this.baseMapper.getPurchaseBomInTransitSum(wrapper);
    }
}
