package com.sd.business.service.sku.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.sku.bo.SkuSpecBo;
import com.sd.business.entity.sku.dto.SkuSpecDto;
import com.sd.business.entity.sku.dto.SkuSpecSelectDto;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.vo.SkuSpecVo;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.sku.SkuSpecMapper;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.sku.SkuSpecService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * sku规格 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-05
 */
@Service
public class SkuSpecServiceImpl extends ServiceImpl<SkuSpecMapper, SkuSpec> implements SkuSpecService {

    @Autowired
    private InventoryService inventoryService;

    @Override
    public Page<SkuSpecVo> getPage(SkuSpecSelectDto dto) {
        IWrapper<SkuSpec> wrapper = getWrapper();
        wrapper.orderByDesc("ss", SkuSpec::getId);
        Page<SkuSpecVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public SkuSpecVo detail(Long id) {
        SkuSpec SkuSpec = this.getById(id);
        SkuSpecVo result = BeanUtil.toBean(SkuSpec, SkuSpecVo.class);
        return result;
    }

    @Override
    public void add(SkuSpecDto skuSpecDto) {
        this.save(skuSpecDto);
    }

    @Override
    public void edit(SkuSpecDto skuSpecDto) {
        this.updateById(skuSpecDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public Map<Long, BomSpecBo> getBomSpecBoByIdList(Collection<Long> bomSpecIdList) {

        if (ObjectUtil.isEmpty(bomSpecIdList)) {
            return Collections.emptyMap();
        }

        bomSpecIdList = bomSpecIdList.stream().filter(ObjectUtil::isNotNull).distinct().collect(Collectors.toList());

        if (ObjectUtil.isEmpty(bomSpecIdList)) {
            return Collections.emptyMap();
        }

        List<BomSpecBo> list = baseMapper.getBomSpecBoList(Wrappers.query().in("bs.id", bomSpecIdList));

        return list.stream().collect(Collectors.toMap(BomSpecBo::getBomSpecId, Function.identity()));
    }

    @Override
    public BomSpecBo getBomSpecBoById(Long bomSpecId) {
        Map<Long, BomSpecBo> map = getBomSpecBoByIdList(Collections.singletonList(bomSpecId));
        return map.get(bomSpecId);
    }

    @Override
    public BigDecimal getSkuInventoryQuantity(Long id) {
        SkuSpec skuSpec = this.getById(id);
        if (skuSpec == null) {
            throw new ServiceException("没有找到sku信息");
        }
        Long bomSpecId = skuSpec.getBomSpecId();
        if (ObjectUtil.isEmpty(bomSpecId)) {
            return BigDecimal.ZERO;
        }
        Map<Long, BomSpecBo> bomSpecBoMap = this.getBomSpecBoByIdList(Collections.singleton(bomSpecId));
        BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpecId);
        if (bomSpecBo == null) {
            throw new ServiceException("未知bom规格id：" + bomSpecId);
        }
        Long warehouseId;
        // 主材
        if (ObjectUtil.equals(bomSpecBo.getClassifyParentId(), 1L)) {
            warehouseId = WarehouseConstant.SEMI_FINISHED_PRODUCT;
        } else {
            warehouseId = WarehouseConstant.PACKAGING_MATERIAL;
        }
        Inventory inventory = inventoryService.getOne(q -> q
                .eq(Inventory::getWarehouseId, warehouseId)
                .eq(Inventory::getBomSpecId, bomSpecId));
        return inventory == null ? BigDecimal.ZERO : inventory.getQuantity();
    }

    @Override
    public Map<Long, SkuSpecBo> getSkuSpecBoByIdList(Collection<Long> skuSpecIdList) {

        if (ObjectUtil.isEmpty(skuSpecIdList)) {
            return Collections.emptyMap();
        }

        skuSpecIdList = skuSpecIdList.stream().filter(ObjectUtil::isNotNull).distinct().collect(Collectors.toList());

        if (ObjectUtil.isEmpty(skuSpecIdList)) {
            return Collections.emptyMap();
        }

        List<SkuSpecBo> list = baseMapper.getSkuSpecBoList(Wrappers.query().in("ss.id", skuSpecIdList));

        return list.stream().collect(Collectors.toMap(SkuSpecBo::getSkuSpecId, Function.identity()));

    }

}
